# coding: utf-8

"""
    Mtmai API

    The Mtmai API

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class OperationEnum(str, Enum):
    """
    可用的操作名称枚举： - **startBlogTask**: 启动单个博客自动化操作任务 - **stopBlogTask**: 停止单个博客自动化操作任务 
    """

    """
    allowed enum values
    """
    STARTBLOGTASK = 'startBlogTask'
    STOPBLOGTASK = 'stopBlogTask'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of OperationEnum from a JSON string"""
        return cls(json.loads(json_str))


