# coding: utf-8

"""
    Mtmai API

    The Mtmai API

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class ArtifactLengthOptions(str, Enum):
    """
    工具内容长度,(文章,代码内容长度)
    """

    """
    allowed enum values
    """
    SHORTEST = 'shortest'
    SHORT = 'short'
    LONG = 'long'
    LONGEST = 'longest'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ArtifactLengthOptions from a JSON string"""
        return cls(json.loads(json_str))


